@@ -1,5 +1,3 @@
-#include "isotree.hpp"
-
 /*    Isolation forests and variations thereof, with adjustments for incorporation
 *     of categorical variables and missing values.
 *     Writen for C++11 standard and aimed at being used in R and Python.
@@ -44,6 +42,7 @@
 *     OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *     OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
+#include "isotree.hpp"
 
 /*  Fit Isolation Forest (or variant) model 
 * 
@@ -121,8 +120,9 @@
 * - sample_size
 *       Sample size of the data sub-samples with which each binary tree will be built. When a terminal node has more than
 *       1 observation, the remaining isolation depth for them is estimated assuming the data and splits are both uniformly
-*       random (separation depth follows a similar process with expected value calculated as in [6]). Recommended value
-*       in [1], [2], [3] is 256, while the default value in the author's code in [5] is 'nrows' here.
+*       random (separation depth follows a similar process with expected value calculated as in [6]). If passing zero,
+*       will set it to 'nrows'. Recommended value in [1], [2], [3] is 256, while the default value in the author's code
+*       in [5] is 'nrows' here.
 * - ntrees
 *       Number of binary trees to build for the model. Recommended value in [1] is 100, while the default value in the
 *       author's code in [5] is 10.
@@ -242,13 +242,17 @@
 *       each category beforehand, even if no observations had that category when fitting the model. Ignored when
 *       passing 'cat_split_type' = 'SingleCateg'.
 * - all_perm
-*       When doing categorical variable splits by pooled gain, whether to consider all possible permutations
-*       of variables to assign to each branch or not. If 'false', will sort the categories by their frequency
-*       and make a grouping in this sorted order. Note that the number of combinations evaluated (if 'true')
-*       is the factorial of the number of present categories in a given column (minus 2). For averaged gain, the
-*       best split is always to put the second most-frequent category in a separate branch, so not evaluating all
-*       permutations (passing 'false') will make it possible to select other splits that respect the sorted frequency order.
-*       Ignored when not using categorical variables or not doing splits by pooled gain.
+*       When doing categorical variable splits by pooled gain with 'ndim=1' (regular model),
+*       whether to consider all possible permutations of variables to assign to each branch or not. If 'false',
+*       will sort the categories by their frequency and make a grouping in this sorted order. Note that the
+*       number of combinations evaluated (if 'true') is the factorial of the number of present categories in
+*       a given column (minus 2). For averaged gain, the best split is always to put the second most-frequent
+*       category in a separate branch, so not evaluating all  permutations (passing 'false') will make it
+*       possible to select other splits that respect the sorted frequency order.
+*       The total number of combinations must be a number that can fit into a 'size_t' variable - for x64-64
+*       systems, this means no column can have more than 20 different categories if using 'all_perm=true',
+*       but note that this is not checked within the function.
+*       Ignored when not using categorical variables or not doing splits by pooled gain or using 'ndim>1'.
 * - imputer (out)
 *       Pointer to already-allocated imputer object, which can be used to produce missing value imputations
 *       in new data. Pass NULL if no missing value imputations are required. Note that this is not related to
@@ -325,7 +329,7 @@ int fit_iforest(IsoForest *model_outputs, ExtIsoForest *model_outputs_ext,
 
     bool calc_dist = tmat != NULL;
 
-    if (calc_dist)
+    if (calc_dist || sample_size == 0)
         sample_size = nrows;
 
     /* put data in structs to shorten function calls */
@@ -802,10 +806,7 @@ void fit_itree(std::vector<IsoTree>    *tree_root,
             }
         }
 
-        if (model_params.missing_action != Fail)
-        {
-            workspace.ext_fill_val.resize(input_data.ncols_tot);
-        }
+        workspace.ext_fill_val.resize(input_data.ncols_tot);
 
     }
 
